﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Encog.Neural.Networks;
using Encog.Neural.Networks.Layers;
using Encog.Engine.Network.Activation;
using Encog.ML.Data;
using Encog.ML.Data.Basic;
using Encog.Util.Arrayutil;
using Encog.App.Quant.Indicators;
using Encog.Util.CSV;
using Encog.App.Quant.Indicators.Predictive;
using Encog.App.Analyst.CSV.Normalize;
using Encog.Util.Simple;
using Encog.Persist;
using Encog.App.Analyst;
using Encog.App.Analyst.Wizard;
using System.Globalization;
using Encog.Util.NetworkUtil;

namespace EncogTrainerNinja
{
    class Program
    {
       
        static void Main(string[] args)
        {
            BasicNetwork _network;
            List<NormalizedField> _stats;
            ProcessIndicators ind = new ProcessIndicators();

            int INPUT_WINDOW = 20;
            int PREDICT_WINDOW = 1;
            int RESULT_WINDOW = 5;
            int HIDDEN1_NEURONS = 12;
            double TARGET_ERROR = 0.01;

            string DIRECTORY = "d:\\";
            string STEP1_FILENAME = Path.Combine(DIRECTORY, "exportfileninja.csv");
            string STEP2_FILENAME = Path.Combine(DIRECTORY, "step2_future.csv");
            string STEP3_FILENAME = Path.Combine(DIRECTORY, "step3_norm.csv");
            string STEP4_FILENAME = Path.Combine(DIRECTORY, "step4_train.csv");
            string STEP5_FILENAME = Path.Combine(DIRECTORY, "step5_network.csv");



            ind.Analyze(new FileInfo(STEP1_FILENAME), true, CSVFormat.English);
            int externalIndicatorCount = ind.Columns.Count - 3;
            ind.AddColumn(new BestReturn(RESULT_WINDOW, true));
            ind.Process(new FileInfo(STEP2_FILENAME));

            var analyst = new EncogAnalyst();
            var wizard = new AnalystWizard(analyst);

            wizard.Wizard(new FileInfo(STEP2_FILENAME), true, AnalystFileFormat.DecpntComma);
            analyst.Script.Normalize.NormalizedFields[0].Action = NormalizationAction.PassThrough;// Date
            analyst.Script.Normalize.NormalizedFields[1].Action = NormalizationAction.PassThrough;// Time

            analyst.Script.Normalize.NormalizedFields[2].Action = NormalizationAction.Normalize; // close
            analyst.Script.Normalize.NormalizedFields[3].Action = NormalizationAction.Normalize;
            analyst.Script.Normalize.NormalizedFields[4].Action = NormalizationAction.Normalize;
            analyst.Script.Normalize.NormalizedFields[5].Action = NormalizationAction.Normalize;
            analyst.Script.Normalize.NormalizedFields[6].Action = NormalizationAction.Normalize;
            analyst.Script.Normalize.NormalizedFields[7].Action = NormalizationAction.Normalize;
            analyst.Script.Normalize.NormalizedFields[8].Action = NormalizationAction.Normalize;
            analyst.Script.Normalize.NormalizedFields[9].Action = NormalizationAction.Normalize;


            var norm = new AnalystNormalizeCSV();

            norm.Analyze(new FileInfo(STEP2_FILENAME), true, CSVFormat.English, analyst);

            norm.ProduceOutputHeaders = true;
            norm.Normalize(new FileInfo(STEP3_FILENAME));


            // neuron counts

            int inputNeurons = INPUT_WINDOW * externalIndicatorCount;
            int outputNeurons = PREDICT_WINDOW;
            Console.WriteLine("inputneurons : {0}", inputNeurons);
            Console.WriteLine("outputNeurons : {0}", outputNeurons);
            //Console.ReadLine();



            // Step 3: Time-box (optional)
            //TemporalWindowCSV
            Console.WriteLine("Step 3: Timebox");
            var twcsv = new TemporalWindowCSV();
            twcsv.Analyze(STEP3_FILENAME, true, CSVFormat.English);
            twcsv.InputWindow = INPUT_WINDOW;
            twcsv.PredictWindow = PREDICT_WINDOW;
            int index = 0;
            twcsv.Fields[index++].Action = TemporalType.Ignore; // date
            twcsv.Fields[index++].Action = TemporalType.Ignore; // time
            twcsv.Fields[index++].Action = TemporalType.Ignore; // close

            for (int i = 0; i < externalIndicatorCount; i++)
                twcsv.Fields[index++].Action = TemporalType.Input; // external indicators

            twcsv.Fields[index++].Action = TemporalType.Predict; // PredictBestReturn
            twcsv.Process(STEP4_FILENAME);

          



            // Step 4: Train neural network

            Console.WriteLine("Step 4: Train");
            
            IMLDataSet training = EncogUtility.LoadCSV2Memory(STEP4_FILENAME, inputNeurons, outputNeurons, true, CSVFormat.English, false);
            BasicNetwork network = new BasicNetwork();
            network.AddLayer(new BasicLayer(new ActivationTANH(), true, inputNeurons));
            network.AddLayer(new BasicLayer(new ActivationTANH(), true, HIDDEN1_NEURONS));
            network.AddLayer(new BasicLayer(new ActivationLinear(), true, outputNeurons));
            network.Structure.FinalizeStructure();
            network.Reset();

            //EncogUtility.TrainToError(network, training, TARGET_ERROR);

            EncogUtility.TrainConsole(network, training, 1);


            // Step 5: Save neural network and stats



            if (!network.Properties.ContainsKey("eval"))
            {
                network.Properties.Add("eval", INPUT_WINDOW.ToString(CultureInfo.InvariantCulture));
            }
            else
            {
                network.Properties["eval"] = INPUT_WINDOW.ToString(CultureInfo.InvariantCulture);
            }

            if (!network.Properties.ContainsKey("predict"))
            {
                network.Properties.Add("predict", PREDICT_WINDOW.ToString(CultureInfo.InvariantCulture));
            }
            else
            {
                network.Properties["predict"] = PREDICT_WINDOW.ToString(CultureInfo.InvariantCulture);
            }
            
            var inf = new FileInfo(STEP5_FILENAME);
            if (inf.Directory != null)
            {
                BasicNetwork tempn = NetworkUtility.SaveNetwork(inf.Directory.ToString(), Path.GetFileName(STEP5_FILENAME), network);
                network = tempn;
            }

            Console.ReadLine();
        }
    }
}
